@@ -2,7 +2,7 @@ utils::globalVariables(c(
   "label_x", "x_min", "x_max", "y_min", "y_max","var",
   "x_pos", "y_pos", "x_offset", "y_offset", "x", "y",
   "present", "combined", "value", "count", "gene", "Contrast","z", "label", "log_p_val",
-  "Celltype", "avg_log2FC", "p_val_adj", "log_p_val_adj","adj_logfc","label_y","label_x","dot_color"
+  "Celltype", "avg_log2FC", "p_val_adj", "log_p_val_adj","adj_logfc","label_y","label_x","dot_color","geom_raster","scale_fill_gradient2","category"
 ))
 
 
@@ -91,7 +91,7 @@ perform_clustering <- function(data, cat_a, cat_b, cat_c) {
     pivot_wider(id_cols = !!sym(cat_a), 
                 names_from = combined, 
                 values_from = value, 
-                values_fill = 0) %>%
+                values_fill = 0L) %>%
     column_to_rownames(cat_a) %>%
     as.matrix()
   
@@ -436,23 +436,22 @@ create_custom_legends <- function(data, cat_c, group, cat_c_colors, group_colors
 }
 
 
-
-#' Create custom legends for the domino plot
+#' Create custom legends for a domino plot
 #'
-#' @param contrast_levels A character vector of contrast level names.
-#' @param var_positions A data frame containing variable positions.
-#' @param var_id A string representing the column name for the variable identifier.
-#' @param contrast A string representing the column name for the contrast variable.
-#' @param logfc_colors A named vector specifying the colors for the log fold change scale.
-#' @param logfc_limits A numeric vector of length 2 specifying the limits for the log fold change color scale.
-#' @param color_scale_name A string specifying the name of the color scale in the legend.
-#' @param size_scale_name A string specifying the name of the size scale in the legend.
-#' @param min_dot_size A numeric value indicating the minimum dot size.
-#' @param max_dot_size A numeric value indicating the maximum dot size.
-#' @param legend_text_size A numeric value indicating the text size for the legend.
+#' @param contrast_levels Character vector of contrast level names.
+#' @param var_positions   Data frame with variable positions.
+#' @param var_id          Column name for the variable identifier.
+#' @param contrast        Column name for the contrast variable.
+#' @param logfc_colors    Named vector with "low", "mid", "high" colours.
+#' @param logfc_limits    Numeric vector (length 2) for logFC scale limits.
+#' @param color_scale_name Title for the logFC colour legend.
+#' @param size_scale_name  Title for the p-value size legend.
+#' @param min_dot_size,max_dot_size Numeric dot-size range.
+#' @param size_limits,size_breaks   Passed to `scale_size_continuous()`.
+#' @param p_label_formatter A function used to format the size legend labels (typically for p-values). Default is `function(lp) sprintf("%.2g", 10^-lp)`.
+#' @param legend_text_size Base font size for legend text.
 #'
-#' @return A ggplot object containing custom legends.
-#' @importFrom ggplot2 geom_raster scale_fill_gradient2
+#' @return A combined `ggplot` object with three aligned legends.
 #' @keywords internal
 create_custom_domino_legends <- function(contrast_levels,
                                          var_positions,
@@ -464,53 +463,217 @@ create_custom_domino_legends <- function(contrast_levels,
                                          size_scale_name,
                                          min_dot_size,
                                          max_dot_size,
-                                         legend_text_size = 8) {
-  # Create a data frame for the color scale legend
-  color_legend_data <- data.frame(
-    x = rep(1, 5),
-    y = 1:5,
-    logfc = seq(from = logfc_limits[1],
-                to = logfc_limits[2],
-                length.out = 5)
+                                         size_limits = NULL,
+                                         size_breaks = NULL,
+                                         legend_text_size = 8,
+                                         p_label_formatter = function(lp) sprintf("%.2g", 10^-lp)
+                                         ) {
+  legend_vars <- var_positions[var_positions[[contrast]] == contrast_levels[1], ]
+  var_legend_data <- data.frame(
+    var = legend_vars[[var_id]],
+    x   = legend_vars$x_offset,
+    y   = legend_vars$y_offset
+  )
+  vars_legend_plot <- ggplot() +
+    geom_point(data = var_legend_data,
+               aes(x, y),
+               size = 3,
+               fill = "white",
+               colour = "black",
+               stroke = .8,
+               shape = 21) +
+    geom_text(data = var_legend_data,
+              aes(
+                x = ifelse(x > 0, x + .15, ifelse(x < 0, x - .15,  0)),
+                y = ifelse(y > 0, y + .15, ifelse(y < 0, y - .15, .15)),
+                label = var,
+                hjust  = ifelse(x < 0, 1, ifelse(x > 0, 0, .5)),
+                vjust  = ifelse(y > 0, 0, ifelse(y < 0, 1, .5))
+              ),
+              size = legend_text_size / 3) +
+    ggtitle("Domino Layout") +
+    theme_void() +
+    theme(
+      plot.margin  = margin(10, 20, 10, 20),
+      plot.background = element_rect(fill = "white", colour = NA),
+      plot.title = element_text(hjust = .5,
+                                size = legend_text_size * 1.5,
+                                face = "bold",
+                                margin = margin(0, 0, 10, 0))
+    ) +
+    coord_fixed(ratio = 1,
+                xlim  = c(-.75, .75),
+                ylim  = c(-.5,  .5),
+                expand = FALSE,
+                clip   = "off")
+  
+  ## 2. logFC colour legend ----
+  grad_data <- expand.grid(
+    x = seq(-.25, .25, length.out = 50),
+    y = seq(  1,   5, length.out = 50)
+  )
+  grad_data$z <- rep(seq(logfc_limits[1], logfc_limits[2], length.out = 50),
+                     each = 50)
+  
+  tick_positions <- seq(1, 5, length.out = 5)
+  tick_labels    <- seq(logfc_limits[1], logfc_limits[2], length.out = 5)
+  tick_data <- data.frame(
+    x     = .35,
+    y     = tick_positions,
+    label = sprintf("%.1f", tick_labels)
   )
   
-  # Create a data frame for the size scale legend
-  size_legend_data <- data.frame(
-    x = rep(1, 5),
-    y = 1:5,
-    log_p_val = seq(from = 0, to = 5, length.out = 5)
-  )
+  logfc_legend_plot <- ggplot() +
+    geom_raster(data = grad_data, aes(x, y, fill = z)) +
+    geom_rect(aes(xmin = -.25, xmax = .25, ymin = 1, ymax = 5),
+              fill = NA, colour = "black", size = .5) +
+    scale_fill_gradient2(
+      low    = logfc_colors["low"],
+      mid    = logfc_colors["mid"],
+      high   = logfc_colors["high"],
+      limits = logfc_limits
+    ) +
+    geom_text(data = tick_data,
+              aes(x, y, label = label),
+              size = legend_text_size / 2.3,
+              hjust = 0, vjust = .5) +
+    ggtitle(color_scale_name) +
+    theme_void() +
+    theme(
+      legend.position = "none",
+      plot.margin  = margin(10, 20, 10, 20),
+      plot.background = element_rect(fill = "white", colour = NA),
+      plot.title = element_text(hjust = .5,
+                                size = legend_text_size * 1.375,
+                                face = "bold",
+                                margin = margin(0, 0, 10, 0))
+    ) +
+    coord_fixed(ratio = 1,
+                xlim  = c(-.75, .75),
+                ylim  = c(.8,  5.2),
+                expand = FALSE,
+                clip   = "off")
   
+  if (is.null(p_label_formatter)) {
+    p_label_formatter <- function(x) x
+  }
+  ## 3. p-value size legend ----
+  legend_breaks <- if (!is.null(size_breaks) && length(size_breaks) >= 3)
+    size_breaks[1:3]
+  else
+    seq(size_limits[1], size_limits[2], length.out = 3)
+  
+  p_val_legend_data <- data.frame(
+    log_p_val = legend_breaks,
+    x         = 0,
+    y         = rev(seq_along(legend_breaks))
+  )
+  print(p_val_legend_data)
+  print(size_scale_name)
+  p_val_legend_plot <- ggplot() +
+    geom_point(data = p_val_legend_data,
+               aes(x, y, size = log_p_val),
+               colour = "black") +
+    geom_point(data = p_val_legend_data,
+               aes(x, y, size = log_p_val),
+               shape = 1, colour = "black") +
+    scale_size_continuous(
+      limits = size_limits,
+      breaks = legend_breaks,
+      range  = c(min_dot_size, max_dot_size)
+    ) +
+    geom_text(data = p_val_legend_data,
+              aes(x = x + .4, y, label = p_label_formatter(log_p_val)),
+              size = legend_text_size / 2.3,
+              hjust = 0) +
+    ggtitle(size_scale_name) +
+    theme_void() +
+    theme(
+      legend.position = "none",
+      plot.margin  = margin(10, 20, 10, 20),
+      plot.background = element_rect(fill = "white", colour = NA),
+      plot.title = element_text(hjust = .5,
+                                size = legend_text_size * 1.375,
+                                face = "bold",
+                                margin = margin(0, 0, 10, 0))
+    ) +
+    coord_fixed(ratio = 1,
+                xlim  = c(-.75, .75),
+                ylim  = c(.5,   3.5),
+                expand = FALSE,
+                clip   = "off")
+  
+  ## 4. Combine ----
+  cowplot::plot_grid(
+    vars_legend_plot,
+    logfc_legend_plot,
+    p_val_legend_plot,
+    ncol = 1,
+    align = "v",
+    axis  = "lr",
+    rel_heights = c(2, 1.5, 1.5)
+  )
+}
+
+
+#' Create custom legends for the domino plot
+#'
+#' @param contrast_levels A character vector of contrast level names.
+#' @param var_positions A data frame containing variable positions.
+#' @param var_id A string representing the column name for the variable identifier.
+#' @param contrast A string representing the column name for the contrast variable.
+#' @param categorical_colors A named vector specifying the colors for each category.
+#' @param color_scale_name A string specifying the name of the color scale in the legend.
+#' @param legend_text_size A numeric value indicating the text size for the legend.
+#' @param left_rect_color A string specifying the color for the left rectangles.
+#' @param right_rect_color A string specifying the color for the right rectangles.
+#'
+#' @return A ggplot object containing custom legends.
+#' @importFrom ggplot2 ggplot geom_point geom_rect geom_text theme_void theme element_text element_rect margin coord_fixed ggtitle geom_raster scale_fill_gradient2
+#' @importFrom cowplot plot_grid
+#' @importFrom stats na.omit
+#' @keywords internal
+create_custom_domino_legends_categorical <- function(contrast_levels,
+                                                   var_positions,
+                                                   var_id,
+                                                   contrast,
+                                                   categorical_colors,
+                                                   color_scale_name,
+                                                   legend_text_size = 8,
+                                                   left_rect_color = "lightblue",
+                                                   right_rect_color = "lightpink") {
   # Get unique variables from the left contrast only (assuming both sides have same variables)
   vars_to_show <- unique(var_positions[var_positions[[contrast]] == contrast_levels[1], var_id])
   
   # Filter var_positions to include only the first contrast
   legend_vars <- var_positions[var_positions[[contrast]] == contrast_levels[1],]
   
-  # Create a simplified legend data frame for the positions exactly as in the example
+  # Create a simplified legend data frame for the positions
   var_legend_data <- data.frame(
     var = legend_vars[[var_id]],
     x = legend_vars$x_offset,
     y = legend_vars$y_offset
   )
   
-  # Create the position legend plot centered - adjust coordinate system to match other legends
+  # Create the position legend plot
   vars_legend_plot <- ggplot() +
-    # Add dots showing variable positions using the correct pattern
+    # Add dots showing variable positions
+    geom_point(
+      data = var_legend_data,
+      aes(x = x, y = y, color = var),
+      size = 3
+    ) +
     geom_point(
       data = var_legend_data,
       aes(x = x, y = y),
-      size = 3,
-      fill = "white",
-      color = "black",
-      stroke = 0.8,
-      shape = 21
+      size = 3.5,
+      shape = 1,
+      color = "black"
     ) +
-    # Add variable labels with proper offset and alignment
+    # Add variable labels
     geom_text(
       data = var_legend_data,
       aes(
-        # Position text with appropriate offsets
         x = ifelse(x > 0, x + 0.15, 
                    ifelse(x < 0, x - 0.15, 
                           ifelse(x == 0, 0, x))),
@@ -518,7 +681,6 @@ create_custom_domino_legends <- function(contrast_levels,
                    ifelse(y < 0, y - 0.15, 
                           ifelse(y == 0 & x == 0, 0.15, y))),
         label = var,
-        # Adjust text alignment based on position
         hjust = ifelse(x < 0, 1,
                        ifelse(x > 0, 0, 0.5)),
         vjust = ifelse(y > 0, 0,
@@ -527,157 +689,310 @@ create_custom_domino_legends <- function(contrast_levels,
       ),
       size = legend_text_size/3
     ) +
-    # Add title
-    ggtitle("Domino Layout") +
+    scale_color_manual(values = categorical_colors) +
     theme_void() +
     theme(
-      plot.margin = margin(10, 20, 10, 20),  # Match margins with other legends
+      plot.margin = margin(10, 20, 10, 20),
       plot.background = element_rect(fill = "white", color = NA),
       plot.title = element_text(hjust = 0.5, size = legend_text_size*1.5, face = "bold", margin = margin(0, 0, 10, 0)),
-      # Force plot to use the entire available space
-      plot.title.position = "plot"
+      legend.position = "none"
     ) +
     coord_fixed(
       ratio = 1,
-      xlim = c(-0.75, 0.75),  # Match x-limits with other legends
+      xlim = c(-0.75, 0.75),
       ylim = c(-0.5, 0.5),
-      expand = FALSE,  # Match with other legends
+      expand = FALSE,
       clip = "off"
-    )
-  
-  # Create data for log fold change legend - centered
-  logfc_range <- seq(from = logfc_limits[1], to = logfc_limits[2], length.out = 5)
-  logfc_legend_data <- data.frame(
-    logfc = logfc_range,
-    x = rep(0, length(logfc_range)),  # Centered at x=0
-    y = seq(length(logfc_range), 1)
-  )
-  
-  # Create a gradient data frame for the color bar - centered
-  grad_data <- expand.grid(
-    x = seq(-0.25, 0.25, length.out = 50),  # Centered around x=0
-    y = seq(1, 5, length.out = 50)
-  )
-  grad_data$z <- rep(seq(logfc_limits[1], logfc_limits[2], length.out = 50), each = 50)
-  
-  # Create tick positions for the color bar - centered
-  tick_positions <- seq(1, 5, length.out = 5)
-  tick_labels <- seq(logfc_limits[1], logfc_limits[2], length.out = 5)
-  tick_data <- data.frame(
-    x = rep(0.35, length(tick_positions)),  # Adjusted for centered bar
-    y = tick_positions,
-    label = sprintf("%.1f", tick_labels)
-  )
-  
-  logfc_legend_plot <- ggplot() +
-    # Add the continuous color bar
-    geom_raster(
-      data = grad_data,
-      aes(x = x, y = y, fill = z)
     ) +
-    # Add border around the color bar
+    ggtitle("Domino Layout")
+  
+  # Create the contrast legend plot
+  contrast_legend_data <- data.frame(
+    contrast = factor(contrast_levels, levels = contrast_levels),
+    x = c(0.5, 1.5),
+    y = c(1, 1)
+  )
+  
+  contrast_legend_plot <- ggplot() +
     geom_rect(
-      aes(xmin = -0.25, xmax = 0.25, ymin = 1, ymax = 5),  # Centered
-      fill = NA, color = "black", size = 0.5
+      data = contrast_legend_data,
+      aes(
+        xmin = x - 0.4,
+        xmax = x + 0.4,
+        ymin = y - 0.4,
+        ymax = y + 0.4,
+        fill = contrast
+      ),
+      color = "darkgrey",
+      alpha = 0.5,
+      linewidth = 0.5
     ) +
-    # Add scale for the fill
-    scale_fill_gradient2(
-      name = color_scale_name,
-      low = logfc_colors["low"],
-      mid = logfc_colors["mid"],
-      high = logfc_colors["high"],
-      limits = logfc_limits
-    ) +
-    # Add tick labels
     geom_text(
-      data = tick_data,
-      aes(x = x, y = y, label = label),
+      data = contrast_legend_data,
+      aes(x = x, y = y + 0.6, label = contrast),
       size = legend_text_size/2.3,
-      hjust = 0,
-      vjust = 0.5
+      hjust = 0.5
+    ) +
+    scale_fill_manual(
+      values = setNames(c(left_rect_color, right_rect_color), contrast_levels)
     ) +
     theme_void() +
     theme(
-      legend.position = "none",
-      plot.margin = margin(10, 20, 10, 20),  # Equal margins
+      plot.margin = margin(10, 20, 10, 20),
       plot.background = element_rect(fill = "white", color = NA),
-      plot.title = element_text(hjust = 0.5, size = legend_text_size*1.375, face = "bold", margin = margin(0, 0, 10, 0))  # Centered title
+      plot.title = element_text(hjust = 0.5, size = legend_text_size*1.375, face = "bold", margin = margin(0, 0, 10, 0)),
+      legend.position = "none"
     ) +
     coord_fixed(
       ratio = 1,
-      xlim = c(-0.75, 0.75),  # Centered x limits
-      ylim = c(0.8, 5.2),
+      xlim = c(0, 2),
+      ylim = c(0.5, 1.5),
+      expand = FALSE,
+      clip = "off"
+    ) +
+    ggtitle("Contrast Groups")
+  
+  # Create the color legend plot
+  color_legend_data <- data.frame(
+    category = factor(names(categorical_colors), levels = names(categorical_colors)),
+    x = 1,
+    y = seq(length(categorical_colors), 1)
+  )
+  
+  color_legend_plot <- ggplot() +
+    geom_point(
+      data = color_legend_data,
+      aes(x = x, y = y, color = category),
+      size = 3
+    ) +
+    geom_point(
+      data = color_legend_data,
+      aes(x = x, y = y),
+      size = 3.5,
+      shape = 1,
+      color = "black"
+    ) +
+    geom_text(
+      data = color_legend_data,
+      aes(x = x + 0.3, y = y, label = category),
+      size = legend_text_size/2.3,
+      hjust = 0
+    ) +
+    scale_color_manual(values = categorical_colors) +
+    theme_void() +
+    theme(
+      plot.margin = margin(10, 20, 10, 20),
+      plot.background = element_rect(fill = "white", color = NA),
+      plot.title = element_text(hjust = 0.5, size = legend_text_size*1.375, face = "bold", margin = margin(0, 0, 10, 0)),
+      legend.position = "none"
+    ) +
+    coord_fixed(
+      ratio = 1,
+      xlim = c(0.8, 2.2),
+      ylim = c(0.5, length(categorical_colors) + 0.5),
       expand = FALSE,
       clip = "off"
     ) +
     ggtitle(color_scale_name)
   
-  # Create data for p-value legend - centered
-  p_val_range <- seq(from = 0, to = 5, length.out = 5)  # -log10(p) from 0 to 5
-  p_val_legend_data <- data.frame(
-    log_p_val = p_val_range,
-    x = rep(0, length(p_val_range)),  # Centered at x=0
-    y = seq(length(p_val_range), 1)
-  )
-  
-  # Calculate a better size range for dots based on the range of p-values
-  size_range <- c(min_dot_size * 1.2, max_dot_size * 0.8)  # Adjusted for better visual scaling
-  
-  # Create p-value legend plot with centered elements
-  p_val_legend_plot <- ggplot() +
-    geom_point(
-      data = p_val_legend_data,
-      aes(x = x, y = y, size = log_p_val),
-      color = "black"
-    ) +
-    geom_point(
-      data = p_val_legend_data,
-      aes(x = x, y = y, size = log_p_val),
-      shape = 1,
-      color = "black"
-    ) +
-    scale_size_continuous(
-      name = size_scale_name,
-      range = size_range
-    ) +
-    theme_void() +
-    theme(
-      legend.position = "none",
-      plot.margin = margin(10, 20, 10, 20),  # Equal margins
-      plot.background = element_rect(fill = "white", color = NA),
-      plot.title = element_text(hjust = 0.5, size = legend_text_size*1.375, face = "bold", margin = margin(0, 0, 10, 0))  # Centered title
-    ) +
-    coord_fixed(
-      ratio = 1,
-      xlim = c(-0.75, 0.75),  # Centered x limits
-      ylim = c(0.5, length(p_val_range) + 0.5),
-      expand = FALSE,
-      clip = "off"
-    ) +
-    geom_text(
-      data = p_val_legend_data,
-      aes(x = x + 0.4, y = y, label = sprintf("%.1f", log_p_val)),  # Adjusted position
-      size = legend_text_size/2.3,  # Size based on legend_text_size
-      color = "black",
-      hjust = 0,
-      vjust = 0.5  # Center vertically
-    ) +
-    ggtitle(size_scale_name)
-  
   # Combine all legend plots
   combined_legend_plot <- cowplot::plot_grid(
     vars_legend_plot,
-    logfc_legend_plot,
-    p_val_legend_plot,
+    contrast_legend_plot,
+    color_legend_plot,
     ncol = 1,
     align = 'v',
-    axis = 'lr',  # Align plots at their left and right edges
-    rel_heights = c(2, 1.5, 1.5)
+    axis = 'lr',
+    rel_heights = c(2, 1, 1.5)
   )
   
   return(combined_legend_plot)
 }
 
+#' Plot Dice Representations on sf Objects
+#'
+#' @description
+#' Creates a ggplot2 layer that places dice representations on spatial features
+#' in an sf object. The dice values are determined by a column in the sf object.
+#'
+#' @param sf_data An sf object containing the spatial features.
+#' @param dice_value_col Character. Name of the column in sf_data containing dice values (1-6).
+#'   Default is "dice".
+#' @param face_color Character vector. Column names in sf_data containing color information
+#'   for each dice dot. If NULL (default), all dots are black.
+#' @param dice_color Character. Background color of the dice. Default is "white".
+#' @param dice_size Numeric. Size of the dice. Default is 3.
+#' @param dot_size Numeric. Size of the dots on the dice. If NULL (default), 
+#'   it's calculated as 20% of dice_size.
+#' @param rectangle_padding Numeric. Padding of the rectangle around the dots, as a 
+#'   proportion of dice_size. Default is 0.05.
+#' @param ... Additional arguments passed to geom_point for the dots.
+#'
+#' @return A list of ggplot2 layers (rectangle layer and dots layer).
+#'
+#' @examples
+#' \dontrun{
+#' library(ggplot2)
+#' library(sf)
+#' 
+#' # Create sample sf data with dice values
+#' nc <- st_read(system.file("shape/nc.shp", package = "sf"))
+#' nc$dice <- sample(1:6, nrow(nc), replace = TRUE)
+#' 
+#' # Basic plot with dice
+#' ggplot(nc) + 
+#'   geom_sf() + 
+#'   geom_dice_sf(sf_data = nc)
+#'   
+#' # Customized dice
+#' ggplot(nc) + 
+#'   geom_sf() + 
+#'   geom_dice_sf(sf_data = nc, dice_color = "lightblue", dice_size = 5)
+#' }
+#'
+#' @importFrom sf st_centroid st_coordinates
+#' @importFrom dplyr %>% group_by summarise
+#' @importFrom ggplot2 geom_rect geom_point aes
+#' @export
+geom_dice_sf <- function(sf_data,
+                         dice_value_col = "dice",
+                         face_color = NULL,
+                         dice_color = "white",
+                         dice_size = 3,
+                         dot_size = NULL,
+                         rectangle_padding = 0.05,
+                         ...) {
+  # Input validation
+  if (!inherits(sf_data, "sf")) {
+    stop("sf_data must be an sf object.", call. = FALSE)
+  }
+  
+  if (!dice_value_col %in% names(sf_data)) {
+    stop(sprintf("Column '%s' not found in sf_data.", dice_value_col), call. = FALSE)
+  }
+  
+  if (!is.null(face_color)) {
+    missing_cols <- face_color[!face_color %in% names(sf_data)]
+    if (length(missing_cols) > 0) {
+      stop(sprintf("Color column(s) not found in sf_data: %s",
+                   paste(missing_cols, collapse = ", ")), 
+           call. = FALSE)
+    }
+  }
+  
+  if (!is.numeric(dice_size) || dice_size <= 0) {
+    stop("dice_size must be a positive number.", call. = FALSE)
+  }
+  
+  if (!is.null(dot_size) && (!is.numeric(dot_size) || dot_size <= 0)) {
+    stop("dot_size must be a positive number or NULL.", call. = FALSE)
+  }
+  
+  if (!is.numeric(rectangle_padding) || rectangle_padding < 0) {
+    stop("rectangle_padding must be a non-negative number.", call. = FALSE)
+  }
+  
+  # Calculate centroids of spatial features
+  centroids <- sf::st_centroid(sf_data)
+  coords <- sf::st_coordinates(centroids)
+  sf_data$centroid_x <- coords[, "X"]
+  sf_data$centroid_y <- coords[, "Y"]
+  
+  # Set scaling parameters
+  dot_scale <- if (!is.null(dot_size)) dot_size else dice_size * 0.2
+  position_scale <- dice_size * 0.1
+  
+  # Generate dice dots for each feature
+  dice_dots <- do.call(rbind, lapply(seq_len(nrow(sf_data)), function(i) {
+    row <- sf_data[i, ]
+    n_dots <- as.numeric(as.character(row[[dice_value_col]]))
+    
+    # Handle custom colors for dots
+    if (!is.null(face_color)) {
+      if (is.na(n_dots) || n_dots < 1 || n_dots > length(face_color)) {
+        warning(sprintf("Dice value %s must be between 1 and %d. Skipping feature %d.",
+                        ifelse(is.na(n_dots), "NA", as.character(n_dots)),
+                        length(face_color), i),
+                call. = FALSE)
+        return(NULL)
+      }
+      palette <- sapply(face_color[seq_len(n_dots)], function(col) row[[col]])
+      dot_colors <- palette
+    } else {
+      if (is.na(n_dots) || n_dots < 1 || n_dots > 6) {
+        warning(sprintf("Dice value %s must be between 1 and 6. Skipping feature %d.",
+                        ifelse(is.na(n_dots), "NA", as.character(n_dots)), i),
+                call. = FALSE)
+        return(NULL)
+      }
+      palette <- rep("black", n_dots)
+      dot_colors <- palette
+    }
+    
+    # Create positions for dots based on dice value
+    positions <- create_var_positions(palette, n_dots)
+    positions$x_offset <- positions$x_offset * position_scale
+    positions$y_offset <- positions$y_offset * position_scale
+    
+    # Center the dots
+    center_x <- mean(positions$x_offset)
+    center_y <- mean(positions$y_offset)
+    positions$x_offset <- positions$x_offset - center_x
+    positions$y_offset <- positions$y_offset - center_y
+    
+    # Calculate final positions
+    positions$x <- row$centroid_x + positions$x_offset
+    positions$y <- row$centroid_y + positions$y_offset
+    positions$feature_id <- i
+    positions$dot_color <- dot_colors
+    
+    positions
+  }))
+  
+  # If no valid dice dots were created, return empty layers
+  if (is.null(dice_dots) || nrow(dice_dots) == 0) {
+    warning("No valid dice representations could be created.", call. = FALSE)
+    return(list(
+      ggplot2::geom_blank(),
+      ggplot2::geom_blank()
+    ))
+  }
+  
+  # Convert to data frame
+  dice_dots <- as.data.frame(dice_dots)
+  
+  # Create rectangles for each dice
+  rect_padding <- dice_size * rectangle_padding
+  group_rects <- dice_dots %>%
+    dplyr::group_by(feature_id) %>%
+    dplyr::summarise(
+      x_min = min(x) - rect_padding,
+      x_max = max(x) + rect_padding,
+      y_min = min(y) - rect_padding,
+      y_max = max(y) + rect_padding,
+      .groups = "drop"
+    )
+  
+  # Create the rectangle layer
+  rect_layer <- ggplot2::geom_rect(
+    data = group_rects,
+    mapping = ggplot2::aes(xmin = x_min, xmax = x_max, ymin = y_min, ymax = y_max),
+    fill = dice_color,
+    color = "black",
+    inherit.aes = FALSE
+  )
+  
+  # Create the dots layer
+  dots_layer <- ggplot2::geom_point(
+    data = dice_dots,
+    mapping = ggplot2::aes(x = x, y = y, color = dot_color),
+    size = dot_scale,
+    ...
+  )
+  
+  # Return both layers
+  list(rect_layer, dots_layer)
+}
 
 
 #' Plot Dice Representations on sf Objects
@@ -863,4 +1178,7 @@ geom_dice_sf <- function(sf_data,
   
   # Return both layers
   list(rect_layer, dots_layer)
-}
\ No newline at end of file
+}
+
+
+
